/**
 * Copyright 2008, Howard Abrams (http://www.howardism.org)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.howardism.example;

/**
 * Has some square properties and methods.
 */
public class Square
{
  int x;
  int y;
  int height;
  int width;

  /**
   * Returns the area associated with this square.
   * @return The area == width * height
   * @throws IllegalStateException If the {@link #setWidth} or {@link #setHeight} 
   *                               properties have not be previously set.
   */
  public int area() throws IllegalStateException
  {
    if ( width < 0 || height < 0 )
      throw new IllegalStateException("The width and height properties must be previously set up.");
    
    return width * height;
  }
  
  /**
   * Returns true if the Square given overlaps with the current square.
   * @param other A reference to another {@link Square} instance
   * @return true is the square given intersects the coordinates of our square.
   */
  public boolean intersect ( Square other ) {
    
     return ! ( other.x              > x+width  || 
                other.x+other.width  < x        ||
                other.y              > y+height ||
                other.y+other.height < y 
            );
  }

  /**
   * Returns the x property.
   * @return Returns the x field.
   */
  public int getX ()
  {
    return x;
  }
  /**
   * Sets the x property.
   * @param x The x to set.
   */
  public void setX (int x)
  {
    this.x = x;
  }
  /**
   * Returns the y property.
   * @return Returns the y field.
   */
  public int getY ()
  {
    return y;
  }
  /**
   * Sets the y property.
   * @param y The y to set.
   */
  public void setY (int y)
  {
    this.y = y;
  }
  /**
   * Returns the height property.
   * @return Returns the height field.
   */
  public int getHeight ()
  {
    return height;
  }
  /**
   * Sets the height property.
   * @param height The height to set.
   */
  public void setHeight (int height)
  {
    this.height = height;
  }
  /**
   * Returns the width property.
   * @return Returns the width field.
   */
  public int getWidth ()
  {
    return width;
  }
  /**
   * Sets the width property.
   * @param width The width to set.
   */
  public void setWidth (int width)
  {
    this.width = width;
  }
  
  public String toString() {
    return "Square["+x+", "+y+", "+(x+width)+", "+(y+height)+" / " + width+"x"+height+"]";
  }
}
