/**
 * Copyright 2008, Howard Abrams (http://www.howardism.org)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.howardism.example

import org.junit.Assert
import org.junit.Test

/**
 * Unit Test for the {@link Square} class.
 */
public class SquareTests extends GroovyTestCase
{
   @Test
   void testArea() {
     def sq = new Square ( width:4, height:3 )
     assert 12 == sq.area()
   }
   
   @Test
   void testBadArea() {
     def sq = new Square ( width: -1, height: 4 )
     shouldFail {
       sq.area()
     }
   }
   
   @Test
   void testPositiveIntersect() {
     def sq1 = new Square ( x:2, y:2, width:2, height:2)
     for ( x in 1..4 ) {
       for ( y in 1..4 ) {
         def sq2 = new Square ( x:x, y:y, width:2, height:2)
         assert sq1.intersect ( sq2 ), "${sq1} <=> ${sq2}"
       }
     }
   }

   @Test
   void testNegativeIntersect() {
     def spots = [ [ 0, 0, 'bottom-left' ],
                   [ 2, 0, 'bottom' ],
                   [ 5, 0, 'bottom-right' ],
                   [ 5, 2, 'right' ],
                   [ 5, 5, 'top-right' ],
                   [ 2, 5, 'top' ],
                   [ 0, 5, 'top-left' ],
                   [ 2, 5, 'left' ]
                 ]
     def sq1 = new Square ( x:2, y:2, width:2, height:2)
     spots.each { spot ->
       def sq2 = new Square ( x:spot[0], y:spot[1], width:1, height:1)
       assert ! sq1.intersect ( sq2 ), "The ${spot[2]} square failed, ${sq2}"
     }
   }
}
